@@ -46,7 +46,7 @@ our @EXPORT = qw(
 	SASS2SCSS_CONVERT_COMMENT
 );
 
-our $VERSION = "v3.0.1_3";
+our $VERSION = "v3.0.2";
 
 require XSLoader;
 XSLoader::load('CSS::Sass', $VERSION);
@@ -118,39 +118,10 @@ sub compile_file {
     wantarray ? ($compiled, $srcmap) : $compiled
 }
 
-sub sass_function_callback {
-    my $cb = shift;
-    my $ret = eval { $cb->(@_) };
-    use Data::Dumper;
-
-
-    unless (UNIVERSAL::isa($ret, "CSS::Sass::Type")) {
-        if (UNIVERSAL::isa($ret, "HASH")) {
-            bless $ret, "CSS::Sass::Type::Map"
-        } elsif (UNIVERSAL::isa($ret, "ARRAY")) {
-            bless $ret, "CSS::Sass::Type::List"
-        } elsif (UNIVERSAL::isa($ret, "REF")) {
-        	die "got refg";
-        }
-    } else {
-    	# we are a sass type, sweet
-    	# warn "pass only by $ret";
-    }
-
-    # warn Dumper $ret;
-
-    return CSS::Sass::Type::Error->new("$@") if $@;
-
-#    return CSS::Sass::Type::Error->new("Perl Sass function returned something that wasn't a CSS::Sass::Type")
-#        unless ref $ret && $ret->isa("CSS::Sass::Type");
-
-    $ret;
-}
-
 1;
 __END__
 
-=head1 NAME
+=head1 NAME - perl bindings for libsass
 
 CSS::Sass - Compile .scss files using libsass
 
@@ -170,7 +141,7 @@ CSS::Sass - Compile .scss files using libsass
   sub foobar { CSS::Sass::Type::String->new('blue') }
   $sass->options->{sass_functions}->{'foobar'} = \ &foobar;
 
-  # Compile string and get css output and source map json
+  # Compile string and get css output and source-map json
   $sass->options->{source_map_file} = 'output.css.map';
   ($css, $srcmap) = $sass->compile('A { color: foobar(); }');
 
@@ -196,7 +167,7 @@ CSS::Sass - Compile .scss files using libsass
   # Functional API
   use CSS::Sass qw(:Default sass_compile);
 
-  # Functional API, with error messages and source map
+  # Functional API, with error messages and source-map
   my ($css, $err, $srcmap) = sass_compile('A { color: red; }');
   die $err if defined $err;
 
@@ -277,7 +248,7 @@ Allows you to inspect or change the options after a call to C<new>.
 
 =item C<($css, $err, $srcmap) = sass_compile(source_code, options)>
 
-Compiles the given Sass source code. It returns CSS, error and source map in
+Compiles the given Sass source code. It returns CSS, error and source-map in
 list context or just the CSS in scalar context. Either CSS or error will be
 C<undef>, but never both.
 
@@ -307,13 +278,25 @@ line the code corresponds to.
 
 =item C<source_map_file>
 
-Setting this option enables the source map generating. The file will not
+Setting this option enables the source-map generating. The file will not
 actually be created, but its content will be returned to the caller. It
-will also enable sourceMappingUrl comment by default. See C<omit_src_map_url>.
+will also enable sourceMappingURL comment by default. See C<omit_src_map_url>.
+
+=item C<source_map_embed>
+
+Embeds the complete source-map content into the sourceMappingURL, by using
+base64 encoded data uri (sourceMappingURL=data:application/json;base64,XXXX)
+
+=item C<source_map_contents>
+
+Embeds the content of each source inside a C<sourcesContent> property in the
+source-map json. Setting this option along with C<source_map_embed> allows
+for a completely self-contained source-map.
 
 =item C<omit_src_map_url>
 
-Set to C<true> to omit the sourceMappingUrl comment from the output css.
+Set to C<true> to omit the sourceMappingURL comment from the output css.
+Setting this options makes C<source_map_embed> useless.
 
 =item C<include_paths>
 
@@ -409,8 +392,6 @@ We bless native return values from custom functions into the correct package.
     # sub get-list { return [ 'foo', 42, 'bar' ] };
     .class { content: nth(get-list(), 2); }
 
-
-
 =back
 
 =head1 MISCELLANEOUS
@@ -470,16 +451,29 @@ L<The CSS::Sass Home Page|https://github.com/sass/perl-libsass>
 
 =head1 AUTHOR
 
-David Caldwell E<lt>david@porkrind.orgE<gt>
+David Caldwell E<lt>david@porkrind.orgE<gt>  
 Marcel Greter E<lt>perl-libsass@ocbnet.chE<gt>
 
-=head1 COPYRIGHT AND LICENSE
+=head1 LICENSE
 
-Copyright (C) 2013 by David Caldwell
-Copyright (C) 2014 by Marcel Greter
+The MIT License (MIT)
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.12.4 or,
-at your option, any later version of Perl 5 you may have available.
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 
 =cut
