@@ -6,6 +6,7 @@
 #include <set>
 #include <algorithm>
 #include <deque>
+#include <unordered_map>
 
 #ifdef __clang__
 
@@ -53,6 +54,7 @@
 #endif
 
 #include "ast_def_macros.hpp"
+#include "inspect.hpp"
 
 #include <sstream>
 #include <iostream>
@@ -62,45 +64,10 @@
 #include "position.hpp"
 #endif
 
+
 namespace Sass {
   using namespace std;
 
-  /////////////////////////////////////////////////////////////////////////////
-  // Mixin class for AST nodes that should behave like vectors. Uses the
-  // "Template Method" design pattern to allow subclasses to adjust their flags
-  // when certain objects are pushed.
-  /////////////////////////////////////////////////////////////////////////////
-  template <typename T>
-  class Vectorized {
-    vector<T> elements_;
-  protected:
-    virtual void adjust_after_pushing(T element) { }
-  public:
-    Vectorized(size_t s = 0) : elements_(vector<T>())
-    { elements_.reserve(s); }
-    virtual ~Vectorized() = 0;
-    size_t length() const   { return elements_.size(); }
-    bool empty() const      { return elements_.empty(); }
-    T& operator[](size_t i) { return elements_[i]; }
-    const T& operator[](size_t i) const { return elements_[i]; }
-    Vectorized& operator<<(T element)
-    {
-      elements_.push_back(element);
-      adjust_after_pushing(element);
-      return *this;
-    }
-    Vectorized& operator+=(Vectorized* v)
-    {
-      for (size_t i = 0, L = v->length(); i < L; ++i) *this << (*v)[i];
-      return *this;
-    }
-    vector<T>& elements() { return elements_; }
-    const vector<T>& elements() const { return elements_; }
-    vector<T>& elements(vector<T>& e) { elements_ = e; return elements_; }
-  };
-  template <typename T>
-  inline Vectorized<T>::~Vectorized() { }
-
   //////////////////////////////////////////////////////////
   // Abstract base class for all abstract syntax tree nodes.
   //////////////////////////////////////////////////////////
@@ -119,6 +86,164 @@ namespace Sass {
   };
   inline AST_Node::~AST_Node() { }
 
+
+  //////////////////////////////////////////////////////////////////////
+  // Abstract base class for expressions. This side of the AST hierarchy
+  // represents elements in value contexts, which exist primarily to be
+  // evaluated and returned.
+  //////////////////////////////////////////////////////////////////////
+  class Expression : public AST_Node {
+  public:
+    enum Concrete_Type {
+      NONE,
+      BOOLEAN,
+      NUMBER,
+      COLOR,
+      STRING,
+      LIST,
+      MAP,
+      NULL_VAL,
+      NUM_TYPES
+    };
+  private:
+    // expressions in some contexts shouldn't be evaluated
+    ADD_PROPERTY(bool, is_delayed);
+    ADD_PROPERTY(bool, is_expanded);
+    ADD_PROPERTY(bool, is_interpolant);
+    ADD_PROPERTY(Concrete_Type, concrete_type);
+  public:
+    Expression(string path, Position position,
+               bool d = false, bool e = false, bool i = false, Concrete_Type ct = NONE)
+    : AST_Node(path, position),
+      is_delayed_(d), is_expanded_(d), is_interpolant_(i), concrete_type_(ct)
+    { }
+    virtual operator bool() { return true; }
+    virtual ~Expression() { };
+    virtual string type() { return ""; /* TODO: raise an error? */ }
+    virtual bool is_invisible() { return false; }
+    static string type_name() { return ""; }
+    virtual bool is_false() { return false; }
+    virtual bool operator==( Expression& rhs) const { return false; }
+    virtual size_t hash() { return 0; }
+  };
+}
+
+
+/////////////////////////////////////////////////////////////////////////////
+// Hash method specializations for unordered_map to work with Sass::Expression
+/////////////////////////////////////////////////////////////////////////////
+
+namespace std {
+  template<>
+  struct hash<Sass::Expression*>
+  {
+    size_t operator()(Sass::Expression* s) const
+    {
+      return s->hash();
+    }
+  };
+  template<>
+  struct equal_to<Sass::Expression*>
+  {
+    bool operator()( Sass::Expression* lhs,  Sass::Expression* rhs) const
+    {
+      return *lhs == *rhs;
+    }
+  };
+}
+
+namespace Sass {
+  using namespace std;
+
+  /////////////////////////////////////////////////////////////////////////////
+  // Mixin class for AST nodes that should behave like vectors. Uses the
+  // "Template Method" design pattern to allow subclasses to adjust their flags
+  // when certain objects are pushed.
+  /////////////////////////////////////////////////////////////////////////////
+  template <typename T>
+  class Vectorized {
+    vector<T> elements_;
+  protected:
+    size_t hash_;
+    void reset_hash() { hash_ = 0; }
+    virtual void adjust_after_pushing(T element) { }
+  public:
+    Vectorized(size_t s = 0) : elements_(vector<T>())
+    { elements_.reserve(s); }
+    virtual ~Vectorized() = 0;
+    size_t length() const   { return elements_.size(); }
+    bool empty() const      { return elements_.empty(); }
+    T& operator[](size_t i) { return elements_[i]; }
+    const T& operator[](size_t i) const { return elements_[i]; }
+    Vectorized& operator<<(T element)
+    {
+      reset_hash();
+      elements_.push_back(element);
+      adjust_after_pushing(element);
+      return *this;
+    }
+    Vectorized& operator+=(Vectorized* v)
+    {
+      for (size_t i = 0, L = v->length(); i < L; ++i) *this << (*v)[i];
+      return *this;
+    }
+    vector<T>& elements() { return elements_; }
+    const vector<T>& elements() const { return elements_; }
+    vector<T>& elements(vector<T>& e) { elements_ = e; return elements_; }
+  };
+  template <typename T>
+  inline Vectorized<T>::~Vectorized() { }
+
+  /////////////////////////////////////////////////////////////////////////////
+  // Mixin class for AST nodes that should behave like ahash table. Uses an
+  // extra <vector> internally to maintain insertion order for interation.
+  /////////////////////////////////////////////////////////////////////////////
+  class Hashed {
+  private:
+    unordered_map<Expression*, Expression*> elements_;
+    vector<Expression*> list_;
+  protected:
+    size_t hash_;
+    void reset_hash() { hash_ = 0; }
+    virtual void adjust_after_pushing(std::pair<Expression*, Expression*> p) { }
+  public:
+    Hashed(size_t s = 0) : elements_(unordered_map<Expression*, Expression*>(s)), list_(vector<Expression*>())
+    { elements_.reserve(s); list_.reserve(s); }
+    virtual ~Hashed();
+    size_t length() const                  { return list_.size(); }
+    bool empty() const                     { return list_.empty(); }
+    bool has(Expression* k) const          { return elements_.count(k) == 1; }
+    Expression* at(Expression* k) const    { return elements_.at(k); }
+    Hashed& operator<<(pair<Expression*, Expression*> p)
+    {
+      reset_hash();
+
+      if (!has(p.first)) list_.push_back(p.first);
+
+      elements_[p.first] = p.second;
+
+      adjust_after_pushing(p);
+      return *this;
+    }
+    Hashed& operator+=(Hashed* h)
+    {
+      if (length() == 0) {
+        this->elements_ = h->elements_;
+        this->list_ = h->list_;
+        return *this;
+      }
+
+      for (auto key : h->keys()) {
+        *this << make_pair(key, h->at(key));
+      }
+      return *this;
+    }
+    const unordered_map<Expression*, Expression*>& pairs() const { return elements_; }
+    const vector<Expression*>& keys() const { return list_; }
+  };
+  inline Hashed::~Hashed() { }
+
+
   /////////////////////////////////////////////////////////////////////////
   // Abstract base class for statements. This side of the AST hierarchy
   // represents elements in expansion contexts, which exist primarily to be
@@ -215,6 +340,20 @@ namespace Sass {
     ATTACH_OPERATIONS();
   };
 
+  ///////////////////
+  // Feature queries.
+  ///////////////////
+  class Feature_Block : public Has_Block {
+    ADD_PROPERTY(Feature_Queries*, feature_queries);
+    ADD_PROPERTY(Selector*, selector);
+  public:
+    Feature_Block(string path, Position position, Feature_Queries* fqs, Block* b)
+    : Has_Block(path, position, b), feature_queries_(fqs), selector_(0)
+    { }
+    bool is_hoistable() { return true; }
+    ATTACH_OPERATIONS();
+  };
+
   ///////////////////////////////////////////////////////////////////////
   // At-rules -- arbitrary directives beginning with "@" that may have an
   // optional statement block.
@@ -350,11 +489,11 @@ namespace Sass {
   // The Sass `@each` control directive.
   //////////////////////////////////////
   class Each : public Has_Block {
-    ADD_PROPERTY(string, variable);
+    ADD_PROPERTY(vector<string>, variables);
     ADD_PROPERTY(Expression*, list);
   public:
-    Each(string path, Position position, string var, Expression* lst, Block* b)
-    : Has_Block(path, position, b), variable_(var), list_(lst)
+    Each(string path, Position position, vector<string> vars, Expression* lst, Block* b)
+    : Has_Block(path, position, b), variables_(vars), list_(lst)
     { }
     ATTACH_OPERATIONS();
   };
@@ -500,49 +639,12 @@ namespace Sass {
     ATTACH_OPERATIONS();
   };
 
-  //////////////////////////////////////////////////////////////////////
-  // Abstract base class for expressions. This side of the AST hierarchy
-  // represents elements in value contexts, which exist primarily to be
-  // evaluated and returned.
-  //////////////////////////////////////////////////////////////////////
-  class Expression : public AST_Node {
-  public:
-    enum Concrete_Type {
-      NONE,
-      BOOLEAN,
-      NUMBER,
-      COLOR,
-      STRING,
-      LIST,
-      MAP,
-      NULL_VAL,
-      NUM_TYPES
-    };
-  private:
-    // expressions in some contexts shouldn't be evaluated
-    ADD_PROPERTY(bool, is_delayed);
-    ADD_PROPERTY(bool, is_interpolant);
-    ADD_PROPERTY(Concrete_Type, concrete_type);
-  public:
-    Expression(string path, Position position,
-               bool d = false, bool i = false, Concrete_Type ct = NONE)
-    : AST_Node(path, position),
-      is_delayed_(d), is_interpolant_(i), concrete_type_(ct)
-    { }
-    virtual operator bool() { return true; }
-    virtual ~Expression() = 0;
-    virtual string type() { return ""; /* TODO: raise an error? */ }
-    virtual bool is_invisible() { return false; }
-    static string type_name() { return ""; }
-    virtual bool is_false() { return false; }
-  };
-  inline Expression::~Expression() { }
-
   ///////////////////////////////////////////////////////////////////////
   // Lists of values, both comma- and space-separated (distinguished by a
   // type-tag.) Also used to represent variable-length argument lists.
   ///////////////////////////////////////////////////////////////////////
   class List : public Expression, public Vectorized<Expression*> {
+    void adjust_after_pushing(Expression* e) { is_expanded(false); }
   public:
     enum Separator { SPACE, COMMA };
   private:
@@ -559,35 +661,81 @@ namespace Sass {
     static string type_name() { return "list"; }
     bool is_invisible() { return !length(); }
     Expression* value_at_index(size_t i);
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        List& l = dynamic_cast<List&>(rhs);
+        if (!(l && length() == l.length() && separator() == l.separator())) return false;
+        for (size_t i = 0, L = l.length(); i < L; ++i)
+          if (!(*(elements()[i]) == *(l[i]))) return false;
+        return true;
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+
+    }
+
+    virtual size_t hash()
+    {
+      if (hash_ > 0) return hash_;
+
+      hash_ = std::hash<string>()(separator() == COMMA ? "comma" : "space");
+
+      for (size_t i = 0, L = length(); i < L; ++i)
+        hash_ ^= (elements()[i])->hash();
+
+      return hash_;
+    }
+
     ATTACH_OPERATIONS();
   };
 
   ///////////////////////////////////////////////////////////////////////
   // Key value paris.
   ///////////////////////////////////////////////////////////////////////
-  class KeyValuePair : public AST_Node {
-    ADD_PROPERTY(Expression*, key);
-    ADD_PROPERTY(Expression*, value);
-  public:
-    KeyValuePair(string p, Position pos,
-              Expression* key = 0, Expression* value = 0)
-    : AST_Node(p, pos), key_(key), value_(value)
-    {
-    }
-    ATTACH_OPERATIONS();
-  };
 
-  class Map : public Expression, public Vectorized<KeyValuePair*> {
+  class Map : public Expression, public Hashed {
+    void adjust_after_pushing(std::pair<Expression*, Expression*> p) { is_expanded(false); }
   public:
     Map(string path, Position position,
          size_t size = 0)
     : Expression(path, position),
-      Vectorized<KeyValuePair*>(size)
+      Hashed(size)
     { concrete_type(MAP); }
     string type() { return "map"; }
     static string type_name() { return "map"; }
     bool is_invisible() { return !length(); }
-    Expression* value_at_index(size_t i);
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        Map& m = dynamic_cast<Map&>(rhs);
+        if (!(m && length() == m.length())) return false;
+        for (auto key : keys())
+          if (!(*at(key) == *m.at(key))) return false;
+        return true;
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+    }
+
+    virtual size_t hash()
+    {
+      if (hash_ > 0) return hash_;
+
+      for (auto key : keys())
+        hash_ ^= key->hash() ^ at(key)->hash();
+
+      return hash_;
+    }
+
     ATTACH_OPERATIONS();
   };
 
@@ -634,6 +782,103 @@ namespace Sass {
     ATTACH_OPERATIONS();
   };
 
+  ////////////////////////////////////////////////////////////
+  // Individual argument objects for mixin and function calls.
+  ////////////////////////////////////////////////////////////
+  class Argument : public Expression {
+    ADD_PROPERTY(Expression*, value);
+    ADD_PROPERTY(string, name);
+    ADD_PROPERTY(bool, is_rest_argument);
+    ADD_PROPERTY(bool, is_keyword_argument);
+    size_t hash_;
+  public:
+    Argument(string p, Position pos, Expression* val, string n = "", bool rest = false, bool keyword = false)
+    : Expression(p, pos), value_(val), name_(n), is_rest_argument_(rest), is_keyword_argument_(keyword), hash_(0)
+    {
+      if (!name_.empty() && is_rest_argument_) {
+        error("variable-length argument may not be passed by name", path(), position());
+      }
+    }
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        Argument& m = dynamic_cast<Argument&>(rhs);
+        if (!(m && name() == m.name())) return false;
+        return *value() == *value();
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+    }
+
+    virtual size_t hash()
+    {
+      if (hash_ > 0) return hash_;
+
+      hash_ = std::hash<string>()(name()) ^ value()->hash();
+
+      return hash_;
+    }
+
+    ATTACH_OPERATIONS();
+  };
+
+  ////////////////////////////////////////////////////////////////////////
+  // Argument lists -- in their own class to facilitate context-sensitive
+  // error checking (e.g., ensuring that all ordinal arguments precede all
+  // named arguments).
+  ////////////////////////////////////////////////////////////////////////
+  class Arguments : public Expression, public Vectorized<Argument*> {
+    ADD_PROPERTY(bool, has_named_arguments);
+    ADD_PROPERTY(bool, has_rest_argument);
+    ADD_PROPERTY(bool, has_keyword_argument);
+  protected:
+    void adjust_after_pushing(Argument* a)
+    {
+      if (!a->name().empty()) {
+        if (has_rest_argument_ || has_keyword_argument_) {
+          error("named arguments must precede variable-length argument", a->path(), a->position());
+        }
+        has_named_arguments_ = true;
+      }
+      else if (a->is_rest_argument()) {
+        if (has_rest_argument_) {
+          error("functions and mixins may only be called with one variable-length argument", a->path(), a->position());
+        }
+        if (has_keyword_argument_) {
+          error("only keyword arguments may follow variable arguments", a->path(), a->position());
+        }
+        has_rest_argument_ = true;
+      }
+      else if (a->is_keyword_argument()) {
+        if (has_keyword_argument_) {
+          error("functions and mixins may only be called with one keyword argument", a->path(), a->position());
+        }
+        has_keyword_argument_ = true;
+      }
+      else {
+        if (has_rest_argument_) {
+          error("ordinal arguments must precede variable-length arguments", a->path(), a->position());
+        }
+        if (has_named_arguments_) {
+          error("ordinal arguments must precede named arguments", a->path(), a->position());
+        }
+      }
+    }
+  public:
+    Arguments(string path, Position position)
+    : Expression(path, position),
+      Vectorized<Argument*>(),
+      has_named_arguments_(false),
+      has_rest_argument_(false),
+      has_keyword_argument_(false)
+    { }
+    ATTACH_OPERATIONS();
+  };
+
   //////////////////
   // Function calls.
   //////////////////
@@ -641,13 +886,43 @@ namespace Sass {
     ADD_PROPERTY(string, name);
     ADD_PROPERTY(Arguments*, arguments);
     ADD_PROPERTY(void*, cookie);
+    size_t hash_;
   public:
     Function_Call(string path, Position position, string n, Arguments* args, void* cookie)
-    : Expression(path, position), name_(n), arguments_(args), cookie_(cookie)
+    : Expression(path, position), name_(n), arguments_(args), cookie_(cookie), hash_(0)
     { concrete_type(STRING); }
     Function_Call(string path, Position position, string n, Arguments* args)
-    : Expression(path, position), name_(n), arguments_(args), cookie_(0)
+    : Expression(path, position), name_(n), arguments_(args), cookie_(0), hash_(0)
     { concrete_type(STRING); }
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        Function_Call& m = dynamic_cast<Function_Call&>(rhs);
+        if (!(m && name() == m.name())) return false;
+        if (!(m && arguments()->length() == m.arguments()->length())) return false;
+        for (size_t i =0, L = arguments()->length(); i < L; ++i)
+          if (!((*arguments())[i] == (*m.arguments())[i])) return false;
+        return true;
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+    }
+
+    virtual size_t hash()
+    {
+      if (hash_ > 0) return hash_;
+
+      hash_ = std::hash<string>()(name());
+      for (auto argument : arguments()->elements())
+        hash_ ^= argument->hash();
+
+      return hash_;
+    }
+
     ATTACH_OPERATIONS();
   };
 
@@ -673,6 +948,25 @@ namespace Sass {
     Variable(string path, Position position, string n)
     : Expression(path, position), name_(n)
     { }
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        Variable& e = dynamic_cast<Variable&>(rhs);
+        return e && name() == e.name();
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+    }
+
+    virtual size_t hash()
+    {
+      return std::hash<string>()(name());
+    }
+
     ATTACH_OPERATIONS();
   };
 
@@ -700,12 +994,14 @@ namespace Sass {
     ADD_PROPERTY(double, value);
     vector<string> numerator_units_;
     vector<string> denominator_units_;
+    size_t hash_;
   public:
     Number(string path, Position position, double val, string u = "")
     : Expression(path, position),
       value_(val),
       numerator_units_(vector<string>()),
-      denominator_units_(vector<string>())
+      denominator_units_(vector<string>()),
+      hash_(0)
     {
       if (!u.empty()) numerator_units_.push_back(u);
       concrete_type(NUMBER);
@@ -714,7 +1010,7 @@ namespace Sass {
     vector<string>& denominator_units() { return denominator_units_; }
     string type() { return "number"; }
     static string type_name() { return "number"; }
-    string unit()
+    string unit() const
     {
       stringstream u;
       for (size_t i = 0, S = numerator_units_.size(); i < S; ++i) {
@@ -793,7 +1089,7 @@ namespace Sass {
       sort(denominator_units_.begin(), denominator_units_.end());
     }
     // useful for making one number compatible with another
-    string find_convertible_unit()
+    string find_convertible_unit() const
     {
       for (size_t i = 0, S = numerator_units_.size(); i < S; ++i) {
         string u(numerator_units_[i]);
@@ -805,6 +1101,28 @@ namespace Sass {
       }
       return string();
     }
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        Number& e(dynamic_cast<Number&>(rhs));
+        if (!e) return false;
+        e.normalize(find_convertible_unit());
+        return unit() == e.unit() && value() == e.value();
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+    }
+
+    virtual size_t hash()
+    {
+      if (hash_ == 0) hash_ = std::hash<double>()(value_);
+      return hash_;
+    }
+
     ATTACH_OPERATIONS();
   };
 
@@ -817,12 +1135,34 @@ namespace Sass {
     ADD_PROPERTY(double, b);
     ADD_PROPERTY(double, a);
     ADD_PROPERTY(string, disp);
+    size_t hash_;
   public:
     Color(string path, Position position, double r, double g, double b, double a = 1, const string disp = "")
-    : Expression(path, position), r_(r), g_(g), b_(b), a_(a), disp_(disp)
+    : Expression(path, position), r_(r), g_(g), b_(b), a_(a), disp_(disp),
+      hash_(0)
     { concrete_type(COLOR); }
     string type() { return "color"; }
     static string type_name() { return "color"; }
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        Color& c = (dynamic_cast<Color&>(rhs));
+        return c && r() == c.r() && g() == c.g() && b() == c.b() && a() == c.a();
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+    }
+
+    virtual size_t hash()
+    {
+      if (hash_ == 0) hash_ = std::hash<double>()(r_) ^ std::hash<double>()(g_) ^ std::hash<double>()(b_) ^ std::hash<double>()(a_);
+      return hash_;
+    }
+
     ATTACH_OPERATIONS();
   };
 
@@ -831,13 +1171,36 @@ namespace Sass {
   ////////////
   class Boolean : public Expression {
     ADD_PROPERTY(bool, value);
+    size_t hash_;
   public:
-    Boolean(string path, Position position, bool val) : Expression(path, position), value_(val)
+    Boolean(string path, Position position, bool val)
+    : Expression(path, position), value_(val),
+      hash_(0)
     { concrete_type(BOOLEAN); }
     virtual operator bool() { return value_; }
     string type() { return "bool"; }
     static string type_name() { return "bool"; }
     virtual bool is_false() { return !value_; }
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        Boolean& e = dynamic_cast<Boolean&>(rhs);
+        return e && value() == e.value();
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+    }
+
+    virtual size_t hash()
+    {
+      if (hash_ == 0) hash_ = std::hash<bool>()(value_);
+      return hash_;
+    }
+
     ATTACH_OPERATIONS();
   };
 
@@ -863,12 +1226,40 @@ namespace Sass {
   ///////////////////////////////////////////////////////////////////////
   class String_Schema : public String, public Vectorized<Expression*> {
     ADD_PROPERTY(char, quote_mark);
+    size_t hash_;
   public:
     String_Schema(string path, Position position, size_t size = 0, bool unq = false, char qm = '\0')
-    : String(path, position, unq), Vectorized<Expression*>(size), quote_mark_(qm)
+    : String(path, position, unq), Vectorized<Expression*>(size), quote_mark_(qm), hash_(0)
     { }
     string type() { return "string"; }
     static string type_name() { return "string"; }
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        String_Schema& e = dynamic_cast<String_Schema&>(rhs);
+        if (!(e && length() == e.length())) return false;
+        for (size_t i = 0, L = length(); i < L; ++i)
+          if (!((*this)[i] == e[i])) return false;
+        return true;
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+    }
+
+    virtual size_t hash()
+    {
+      if (hash_ > 0) return hash_;
+
+      for (auto string : elements())
+        hash_ ^= string->hash();
+
+      return hash_;
+    }
+
     ATTACH_OPERATIONS();
   };
 
@@ -877,21 +1268,43 @@ namespace Sass {
   ////////////////////////////////////////////////////////
   class String_Constant : public String {
     ADD_PROPERTY(string, value);
+    string unquoted_;
+    size_t hash_;
   public:
     String_Constant(string path, Position position, string val, bool unq = false)
-    : String(path, position, unq, true), value_(val)
-    { }
+    : String(path, position, unq, true), value_(val), hash_(0)
+    { unquoted_ = unquote(value_); }
     String_Constant(string path, Position position, const char* beg, bool unq = false)
-    : String(path, position, unq, true), value_(string(beg))
-    { }
+    : String(path, position, unq, true), value_(string(beg)), hash_(0)
+    { unquoted_ = unquote(value_); }
     String_Constant(string path, Position position, const char* beg, const char* end, bool unq = false)
-    : String(path, position, unq, true), value_(string(beg, end-beg))
-    { }
+    : String(path, position, unq, true), value_(string(beg, end-beg)), hash_(0)
+    { unquoted_ = unquote(value_); }
     String_Constant(string path, Position position, const Token& tok, bool unq = false)
-    : String(path, position, unq, true), value_(string(tok.begin, tok.end))
-    { }
+    : String(path, position, unq, true), value_(string(tok.begin, tok.end)), hash_(0)
+    { unquoted_ = unquote(value_); }
     string type() { return "string"; }
     static string type_name() { return "string"; }
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      try
+      {
+        String_Constant& e = dynamic_cast<String_Constant&>(rhs);
+        return e && unquoted_ == e.unquoted_;
+      }
+      catch (std::bad_cast&)
+      {
+        return false;
+      }
+    }
+
+    virtual size_t hash() const
+    {
+      if (hash_ == 0) std::hash<string>()(unquoted_);
+      return hash_;
+    }
+
     bool is_quoted() { return value_.length() && (value_[0] == '"' || value_[0] == '\''); }
     char quote_mark() { return is_quoted() ? value_[0] : '\0'; }
     ATTACH_OPERATIONS();
@@ -929,6 +1342,50 @@ namespace Sass {
     ATTACH_OPERATIONS();
   };
 
+  ///////////////////
+  // Feature queries.
+  ///////////////////
+  class Feature_Queries : public Expression, public Vectorized<Feature_Query*> {
+  public:
+    Feature_Queries(string path, Position position, size_t s = 0)
+    : Expression(path, position), Vectorized<Feature_Query*>(s)
+    { }
+    ATTACH_OPERATIONS();
+  };
+
+  /////////////////
+  // Feature query.
+  /////////////////
+  class Feature_Query : public Expression, public Vectorized<Feature_Query_Condition*> {
+    ADD_PROPERTY(bool, is_negated);
+  public:
+    Feature_Query(string path, Position position, size_t s = 0, bool n = false)
+    : Expression(path, position), Vectorized<Feature_Query_Condition*>(s),
+      is_negated_(false)
+    { }
+    ATTACH_OPERATIONS();
+  };
+
+  ////////////////////////////////////////////////////////
+  // Feature expressions (for use inside feature queries).
+  ////////////////////////////////////////////////////////
+  class Feature_Query_Condition : public Expression {
+  public:
+    enum Operand { NONE, AND, OR };
+  private:
+    ADD_PROPERTY(Expression*, feature);
+    ADD_PROPERTY(Expression*, value);
+    ADD_PROPERTY(Operand, operand);
+    ADD_PROPERTY(bool, is_negated);
+  public:
+    Feature_Query_Condition(string path, Position position,
+                           Expression* f, Expression* v,
+                           Operand o = NONE, bool n = false, bool i = false)
+    : Expression(path, position), feature_(f), value_(v), operand_(o), is_negated_(n)
+    { }
+    ATTACH_OPERATIONS();
+  };
+
   //////////////////
   // The null value.
   //////////////////
@@ -940,6 +1397,17 @@ namespace Sass {
     bool is_invisible() { return true; }
     operator bool() { return false; }
     bool is_false() { return true; }
+
+    virtual bool operator==(Expression& rhs) const
+    {
+      return rhs.concrete_type() == NULL_VAL;
+    }
+
+    virtual size_t hash()
+    {
+      return 0;
+    }
+
     ATTACH_OPERATIONS();
   };
 
@@ -1019,80 +1487,11 @@ namespace Sass {
     ATTACH_OPERATIONS();
   };
 
-  ////////////////////////////////////////////////////////////
-  // Individual argument objects for mixin and function calls.
-  ////////////////////////////////////////////////////////////
-  class Argument : public Expression {
-    ADD_PROPERTY(Expression*, value);
-    ADD_PROPERTY(string, name);
-    ADD_PROPERTY(bool, is_rest_argument);
-  public:
-    Argument(string p, Position pos, Expression* val, string n = "", bool rest = false)
-    : Expression(p, pos), value_(val), name_(n), is_rest_argument_(rest)
-    {
-      if (!name_.empty() && is_rest_argument_) {
-        error("variable-length argument may not be passed by name", path(), position());
-      }
-    }
-    ATTACH_OPERATIONS();
-  };
-
   //////////////////////////////////////////////////////////////////////////////////////////
   // Additional method on Lists to retrieve values directly or from an encompassed Argument.
   //////////////////////////////////////////////////////////////////////////////////////////
   inline Expression* List::value_at_index(size_t i) { return is_arglist_ ? ((Argument*)(*this)[i])->value() : (*this)[i]; }
 
-
-  //////////////////////////////////////////////////////////////////////////////////////////
-  // Additional method on Maps to retrieve values directly.
-  //////////////////////////////////////////////////////////////////////////////////////////
-  inline Expression* Map::value_at_index(size_t i) { return (*this)[i]->value(); }
-
-  ////////////////////////////////////////////////////////////////////////
-  // Argument lists -- in their own class to facilitate context-sensitive
-  // error checking (e.g., ensuring that all ordinal arguments precede all
-  // named arguments).
-  ////////////////////////////////////////////////////////////////////////
-  class Arguments : public Expression, public Vectorized<Argument*> {
-    ADD_PROPERTY(bool, has_named_arguments);
-    ADD_PROPERTY(bool, has_rest_argument);
-  protected:
-    void adjust_after_pushing(Argument* a)
-    {
-      if (!a->name().empty()) {
-        if (has_rest_argument_) {
-          error("named arguments must precede variable-length argument", a->path(), a->position());
-        }
-        has_named_arguments_ = true;
-      }
-      else if (a->is_rest_argument()) {
-        if (has_rest_argument_) {
-          error("functions and mixins may only be called with one variable-length argument", a->path(), a->position());
-        }
-        if (has_named_arguments_) {
-          error("functions and mixins may not be called with both named arguments and variable-length arguments", a->path(), a->position());
-        }
-        has_rest_argument_ = true;
-      }
-      else {
-        if (has_rest_argument_) {
-          error("ordinal arguments must precede variable-length arguments", a->path(), a->position());
-        }
-        if (has_named_arguments_) {
-          error("ordinal arguments must precede named arguments", a->path(), a->position());
-        }
-      }
-    }
-  public:
-    Arguments(string path, Position position)
-    : Expression(path, position),
-      Vectorized<Argument*>(),
-      has_named_arguments_(false),
-      has_rest_argument_(false)
-    { }
-    ATTACH_OPERATIONS();
-  };
-
   /////////////////////////////////////////
   // Abstract base class for CSS selectors.
   /////////////////////////////////////////
@@ -1133,10 +1532,10 @@ namespace Sass {
     virtual ~Simple_Selector() = 0;
     virtual Compound_Selector* unify_with(Compound_Selector*, Context&);
     virtual bool is_pseudo_element() { return false; }
-    
+
     bool operator==(const Simple_Selector& rhs) const;
     inline bool operator!=(const Simple_Selector& rhs) const { return !(*this == rhs); }
-    
+
     bool operator<(const Simple_Selector& rhs) const;
   };
   inline Simple_Selector::~Simple_Selector() { }
@@ -1251,7 +1650,7 @@ namespace Sass {
         return true;
       }
       else {
-      	// If it's not a known pseudo-element, check whether it looks like one. This is similar to the type method on the Pseudo class in ruby sass.
+        // If it's not a known pseudo-element, check whether it looks like one. This is similar to the type method on the Pseudo class in ruby sass.
         return name().find("::") == 0;
       }
     }
@@ -1271,7 +1670,7 @@ namespace Sass {
     { }
     ATTACH_OPERATIONS();
   };
-  
+
   struct Complex_Selector_Pointer_Compare {
     bool operator() (const Complex_Selector* const pLeft, const Complex_Selector* const pRight) const;
   };
@@ -1300,7 +1699,7 @@ namespace Sass {
     virtual Selector_Placeholder* find_placeholder();
     Simple_Selector* base()
     {
-    	// Implement non-const in terms of const. Safe to const_cast since this method is non-const
+      // Implement non-const in terms of const. Safe to const_cast since this method is non-const
       return const_cast<Simple_Selector*>(static_cast<const Compound_Selector*>(this)->base());
     }
     const Simple_Selector* base() const {
@@ -1323,16 +1722,16 @@ namespace Sass {
              !static_cast<Selector_Reference*>((*this)[0])->selector();
     }
     vector<string> to_str_vec(); // sometimes need to convert to a flat "by-value" data structure
-    
+
     bool operator<(const Compound_Selector& rhs) const;
-    
+
     bool operator==(const Compound_Selector& rhs) const;
     inline bool operator!=(const Compound_Selector& rhs) const { return !(*this == rhs); }
 
     SourcesSet& sources() { return sources_; }
     void clearSources() { sources_.clear(); }
     void mergeSources(SourcesSet& sources, Context& ctx);
-    
+
     Compound_Selector* clone(Context&) const; // does not clone the Simple_Selector*s
 
     Compound_Selector* minus(Compound_Selector* rhs, Context& ctx);
@@ -1388,15 +1787,15 @@ namespace Sass {
       //s
 
       SourcesSet srcs;
-      
+
       Compound_Selector* pHead = head();
       Complex_Selector*  pTail = tail();
-      
+
       if (pHead) {
         SourcesSet& headSources = pHead->sources();
         srcs.insert(headSources.begin(), headSources.end());
       }
-      
+
       if (pTail) {
         SourcesSet tailSources = pTail->sources();
         srcs.insert(tailSources.begin(), tailSources.end());
@@ -1409,11 +1808,11 @@ namespace Sass {
       Complex_Selector* pIter = this;
       while (pIter) {
         Compound_Selector* pHead = pIter->head();
-        
+
         if (pHead) {
           pHead->mergeSources(sources, ctx);
         }
-        
+
         pIter = pIter->tail();
       }
     }
@@ -1421,11 +1820,11 @@ namespace Sass {
       Complex_Selector* pIter = this;
       while (pIter) {
         Compound_Selector* pHead = pIter->head();
-        
+
         if (pHead) {
           pHead->clearSources();
         }
-        
+
         pIter = pIter->tail();
       }
     }
@@ -1434,8 +1833,8 @@ namespace Sass {
     vector<Compound_Selector*> to_vector();
     ATTACH_OPERATIONS();
   };
-  
-	typedef deque<Complex_Selector*> ComplexSelectorDeque;
+
+  typedef deque<Complex_Selector*> ComplexSelectorDeque;
 
   ///////////////////////////////////
   // Comma-separated selector groups.
@@ -1462,11 +1861,11 @@ namespace Sass {
     // vector<Complex_Selector*> members() { return elements_; }
     ATTACH_OPERATIONS();
   };
-  
-  
+
+
   template<typename SelectorType>
   bool selectors_equal(const SelectorType& one, const SelectorType& two, bool simpleSelectorOrderDependent) {
-  	// Test for equality among selectors while differentiating between checks that demand the underlying Simple_Selector
+    // Test for equality among selectors while differentiating between checks that demand the underlying Simple_Selector
     // ordering to be the same or not. This works because operator< (which doesn't make a whole lot of sense for selectors, but
     // is required for proper stl collection ordering) is implemented using string comparision. This gives stable sorting
     // behavior, and can be used to determine if the selectors would have exactly idential output. operator== matches the
@@ -1475,16 +1874,15 @@ namespace Sass {
     //
     // Due to the reliance on operator== and operater< behavior, this templated method is currently only intended for
     // use with Compound_Selector and Complex_Selector objects.
-  	if (simpleSelectorOrderDependent) {
-    	return !(one < two) && !(two < one);
+    if (simpleSelectorOrderDependent) {
+      return !(one < two) && !(two < one);
     } else {
-    	return one == two;
+      return one == two;
     }
   }
 
 }
 
-
 #ifdef __clang__
 
 #pragma clang diagnostic pop
