@@ -84,7 +84,7 @@ namespace Sass {
       if (val) return val;
 
       List* lval = dynamic_cast<List*>(env[argname]);
-      if (lval && lval->length() == 0) return new (ctx.mem) Map(path, position, 1);
+      if (lval && lval->length() == 0) return new (ctx.mem) Map(path, position, 0);
 
       // fallback on get_arg for error handling
       val = get_arg<Map>(argname, env, sig, path, position, backtrace);
@@ -876,9 +876,9 @@ namespace Sass {
         size_t start = UTF_8::offset_at_position(str, UTF_8::normalize_index(n->value(), UTF_8::code_point_count(str)));
         size_t end = UTF_8::offset_at_position(str, UTF_8::normalize_index(m->value(), UTF_8::code_point_count(str)));
 
-        if(start - end == 0) {
-          newstr = str.substr(start, end - start);
-        } else {
+        if(start == end) {
+          newstr = str.substr(start, 1);
+        } else if(end > start) {
           newstr = str.substr(start, end - start + UTF_8::code_point_size_at_offset(str, end));
         }
         if(quotemark) {
@@ -1052,6 +1052,26 @@ namespace Sass {
       return l->value_at_index(index);
     }
 
+    Signature set_nth_sig = "set-nth($list, $n, $value)";
+    BUILT_IN(set_nth)
+    {
+      List* l = dynamic_cast<List*>(env["$list"]);
+      Number* n = ARG("$n", Number);
+      Expression* v = ARG("$value", Expression);
+      if (!l) {
+        l = new (ctx.mem) List(path, position, 1);
+        *l << ARG("$list", Expression);
+      }
+      if (l->empty()) error("argument `$list` of `" + string(sig) + "` must not be empty", path, position);
+      double index = std::floor(n->value() < 0 ? l->length() + n->value() : n->value() - 1);
+      if (index < 0 || index > l->length() - 1) error("index out of bounds for `" + string(sig) + "`", path, position);
+      List* result = new (ctx.mem) List(path, position, l->length(), l->separator());
+      for (size_t i = 0, L = l->length(); i < L; ++i) {
+        *result << ((i == index) ? v : (*l)[i]);
+      }
+      return result;
+    }
+
     Signature index_sig = "index($list, $value)";
     BUILT_IN(index)
     {
@@ -1190,11 +1210,11 @@ namespace Sass {
     {
       Map* m = ARGM("$map", Map, ctx);
       Expression* v = ARG("$key", Expression);
-      if (!m || m->empty()) return new (ctx.mem) Null(path, position);
-      for (size_t i = 0, L = m->length(); i < L; ++i) {
-        if (eq((*m)[i]->key(), v, ctx)) return m->value_at_index(i);
+      try {
+        return m->at(v);
+      } catch (const std::out_of_range&) {
+        return new (ctx.mem) Null(path, position);
       }
-      return new (ctx.mem) Null(path, position);
     }
 
     Signature map_has_key_sig = "map-has-key($map, $key)";
@@ -1202,21 +1222,16 @@ namespace Sass {
     {
       Map* m = ARGM("$map", Map, ctx);
       Expression* v = ARG("$key", Expression);
-      if (!m || m->empty()) return new (ctx.mem) Boolean(path, position, false);
-      for (size_t i = 0, L = m->length(); i < L; ++i) {
-        if (eq((*m)[i]->key(), v, ctx)) return new (ctx.mem) Boolean(path, position, true);
-      }
-      return new (ctx.mem) Boolean(path, position, false);
+      return new (ctx.mem) Boolean(path, position, m->has(v));
     }
 
     Signature map_keys_sig = "map-keys($map)";
     BUILT_IN(map_keys)
     {
       Map* m = ARGM("$map", Map, ctx);
-      List* result = new (ctx.mem) List(path, position, 1, List::COMMA);
-      if (!m || m->empty()) return result;
-      for (size_t i = 0, L = m->length(); i < L; ++i) {
-        *result << (*m)[i]->key();
+      List* result = new (ctx.mem) List(path, position, m->length(), List::COMMA);
+      for ( auto key : m->keys()) {
+        *result << key;
       }
       return result;
     }
@@ -1225,10 +1240,9 @@ namespace Sass {
     BUILT_IN(map_values)
     {
       Map* m = ARGM("$map", Map, ctx);
-      List* result = new (ctx.mem) List(path, position, 1, List::COMMA);
-      if (!m || m->empty()) return result;
-      for (size_t i = 0, L = m->length(); i < L; ++i) {
-        *result << (*m)[i]->value();
+      List* result = new (ctx.mem) List(path, position, m->length(), List::COMMA);
+      for ( auto key : m->keys()) {
+        *result << m->at(key);
       }
       return result;
     }
@@ -1253,12 +1267,12 @@ namespace Sass {
       Map* m = ARGM("$map", Map, ctx);
       List* arglist = ARG("$keys", List);
       Map* result = new (ctx.mem) Map(path, position, 1);
-      for (size_t i = 0, L = m->length(); i < L; ++i) {
+      for (auto key : m->keys()) {
         remove = false;
         for (size_t j = 0, K = arglist->length(); j < K && !remove; ++j) {
-          remove = eq((*m)[i]->key(), arglist->value_at_index(j), ctx);
+          remove = eq(key, arglist->value_at_index(j), ctx);
         }
-        if (!remove) *result << (*m)[i];
+        if (!remove) *result << make_pair(key, m->at(key));
       }
       return result;
     }
@@ -1271,10 +1285,8 @@ namespace Sass {
       for (size_t i = 0, L = arglist->length(); i < L; ++i) {
         string name = string(((Argument*)(*arglist)[i])->name());
         string sanitized_name = string(name, 1);
-        *result << new (ctx.mem) KeyValuePair(path,
-                                              position,
-                                              new (ctx.mem) String_Constant(path, position, sanitized_name),
-                                              ((Argument*)(*arglist)[i])->value());
+        *result << make_pair(new (ctx.mem) String_Constant(path, position, sanitized_name),
+                             ((Argument*)(*arglist)[i])->value());
       }
       return result;
     }
