@@ -92,7 +92,7 @@ namespace Sass {
         if((p = escape(src))) {
           src = p;
           continue;
-        } 
+        }
         else if((p = exactly<'"'>(src))) {
           return p;
         }
@@ -111,7 +111,7 @@ namespace Sass {
         if((p = escape(src))) {
           src = p;
           continue;
-        } 
+        }
         else if((p = exactly<'\''>(src))) {
           return p;
         }
@@ -166,16 +166,6 @@ namespace Sass {
                                                 backslash_something > > >(src);
     }
 
-    const char* map_key(const char* src) {
-      return sequence< spaces_and_comments,
-                       one_plus< sequence< exactly<'('>, spaces_and_comments > >,
-                       one_plus< alternatives< identifier, alnums, string_constant, variable, spaces > >,
-                       spaces_and_comments,
-                       zero_plus< exactly<')'> >,
-                       spaces_and_comments,
-                       exactly<':'> >(src);
-    }
-
     // Match CSS css variables.
     const char* custom_property_name(const char* src) {
       return sequence< exactly<'-'>, exactly<'-'>, identifier >(src);
@@ -217,6 +207,10 @@ namespace Sass {
       return exactly<media_kwd>(src);
     }
 
+    const char* supports(const char* src) {
+      return exactly<supports_kwd>(src);
+    }
+
     const char* keyframes(const char* src) {
       return sequence< exactly<'@'>, optional< vendor_prefix >, exactly< keyframes_kwd > >(src);
     }
@@ -298,7 +292,8 @@ namespace Sass {
     const char* name(const char* src) {
       return one_plus< alternatives< alnum,
                                      exactly<'-'>,
-                                     exactly<'_'> > >(src);
+                                     exactly<'_'>,
+                                     exactly<'\\'> > >(src);
     }
 
     const char* warn(const char* src) {
@@ -337,7 +332,7 @@ namespace Sass {
     }
     // Attribute name in an attribute selector.
     const char* attribute_name(const char* src) {
-      return alternatives< sequence< optional<namespace_prefix>, identifier>, 
+      return alternatives< sequence< optional<namespace_prefix>, identifier>,
                            identifier >(src);
     }
     // match placeholder selectors
@@ -479,7 +474,7 @@ namespace Sass {
 
     // Match SCSS variable names.
     const char* variable(const char* src) {
-      return sequence<exactly<'$'>, name>(src);
+      return sequence<exactly<'$'>, identifier>(src);
     }
 
     // Match Sass boolean keywords.
