@@ -23,8 +23,10 @@ struct Sass_Context {
 
   int          output_style;
   bool         source_comments;
-  const char*  source_map_file;
+  bool         source_map_embed;
+  bool         source_map_contents;
   bool         omit_source_map_url;
+  const char*  source_map_file;
   const char*  image_path;
   const char*  output_path;
   const char*  include_paths_string;
